/*
 * linux/arch/arm/mach-omap2/board-omap3_sbc8110.c
 *
 * Copyright (C) 2009 Embest
 *
 * Modified from mach-omap2/board-omap3beagle.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/delay.h>
#include <linux/err.h>
#include <linux/clk.h>
#include <linux/io.h>
#include <linux/leds.h>
#include <linux/gpio.h>
#include <linux/input.h>
#include <linux/gpio_keys.h>

#include <linux/mtd/mtd.h>
#include <linux/mtd/partitions.h>
#include <linux/mtd/nand.h>

#include <linux/regulator/machine.h>

#include <mach/hardware.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/flash.h>

#include <mach/board.h>
#include <mach/usb-musb.h>
#include <mach/usb-ehci.h>
#include <mach/common.h>
#include <mach/gpmc.h>
#include <mach/nand.h>
#include <mach/mux.h>

#include "twl4030-generic-scripts.h"
#include "mmc-twl4030.h"

#include <mach/keypad.h>
#include <mach/mcspi.h>
#include <mach/display.h>
#include <linux/dm9000.h>
#include <linux/interrupt.h>
#include <linux/input.h>
#include <linux/spi/spi.h>
#include <linux/spi/ads7846.h>
#include <linux/w1-gpio.h>

#define GPMC_CS0_BASE  0x60
#define GPMC_CS_SIZE   0x30

#define NAND_BLOCK_SIZE		SZ_128K

static struct mtd_partition omap3_sbc8110_nand_partitions[] = {
	/* All the partition sizes are listed in terms of NAND block size */
	{
		.name		= "X-Loader",
		.offset		= 0,
		.size		= 4 * NAND_BLOCK_SIZE,
		.mask_flags	= MTD_WRITEABLE,	/* force read-only */
	},
	{
		.name		= "U-Boot",
		.offset		= MTDPART_OFS_APPEND,	/* Offset = 0x80000 */
		.size		= 15 * NAND_BLOCK_SIZE,
		.mask_flags	= MTD_WRITEABLE,	/* force read-only */
	},
	{
		.name		= "U-Boot Env",
		.offset		= MTDPART_OFS_APPEND,	/* Offset = 0x260000 */
		.size		= 1 * NAND_BLOCK_SIZE,
	},
	{
		.name		= "Kernel",
		.offset		= MTDPART_OFS_APPEND,	/* Offset = 0x280000 */
		.size		= 32 * NAND_BLOCK_SIZE,
	},
	{
		.name		= "File System",
		.offset		= MTDPART_OFS_APPEND,	/* Offset = 0x680000 */
		.size		= MTDPART_SIZ_FULL,
	},
};

static struct omap_nand_platform_data omap3_sbc8110_nand_data = {
	.options	= NAND_BUSWIDTH_16,
	.parts		= omap3_sbc8110_nand_partitions,
	.nr_parts	= ARRAY_SIZE(omap3_sbc8110_nand_partitions),
	.dma_channel	= -1,		/* disable DMA in OMAP NAND driver */
	.nand_setup	= NULL,
	.dev_ready	= NULL,
};

static struct resource omap3_sbc8110_nand_resource = {
	.flags		= IORESOURCE_MEM,
};

static struct platform_device omap3_sbc8110_nand_device = {
	.name		= "omap2-nand",
	.id		= -1,
	.dev		= {
		.platform_data	= &omap3_sbc8110_nand_data,
	},
	.num_resources	= 1,
	.resource	= &omap3_sbc8110_nand_resource,
};

#include "sdram-micron-mt46h32m32lf-6.h"

static struct omap_uart_config omap3_sbc8110_uart_config __initdata = {
	.enabled_uarts	= ((1 << 0) | (1 << 1) | (1 << 2)),
};

static struct twl4030_usb_data sbc8110_usb_data = {
	.usb_mode	= T2_USB_MODE_ULPI,
};

static struct twl4030_hsmmc_info mmc[] = {
	{
		.mmc		= 1,
		.wires		= 8,
		.gpio_wp	= 29,
	},
	{}	/* Terminator */
};

static struct gpio_led gpio_leds[];

static int sbc8110_twl_gpio_setup(struct device *dev,
		unsigned gpio, unsigned ngpio)
{
	/* gpio + 0 is "mmc0_cd" (input/IRQ) */
	omap_cfg_reg(AH8_34XX_GPIO29);
	mmc[0].gpio_cd = gpio + 0;
	twl4030_mmc_init(mmc);

	/* REVISIT: need ehci-omap hooks for external VBUS
	 * power switch and overcurrent detect
	 */

	gpio_request(gpio + 1, "EHCI_nOC");
	gpio_direction_input(gpio + 1);

	/* TWL4030_GPIO_MAX + 0 == ledA, EHCI nEN_USB_PWR (out, active low) */
	gpio_request(gpio + TWL4030_GPIO_MAX, "nEN_USB_PWR");
	gpio_direction_output(gpio + TWL4030_GPIO_MAX, 1);

	/* TWL4030_GPIO_MAX + 1 == ledB, PMU_STAT (out, active low LED) */
//	gpio_leds[2].gpio = gpio + TWL4030_GPIO_MAX + 1;

	return 0;
}

static struct twl4030_gpio_platform_data sbc8110_gpio_data = {
	.gpio_base	= OMAP_MAX_GPIO_LINES,
	.irq_base	= TWL4030_GPIO_IRQ_BASE,
	.irq_end	= TWL4030_GPIO_IRQ_END,
	.use_leds	= true,
	.pullups	= BIT(1),
	.pulldowns	= BIT(2) | BIT(6) | BIT(7) | BIT(8) | BIT(13)
				| BIT(15) | BIT(16) | BIT(17),
	.setup		= sbc8110_twl_gpio_setup,
};

/* VMMC1 for MMC1 pins CMD, CLK, DAT0..DAT3 (20 mA, plus card == max 220 mA) */
static struct regulator_init_data sbc8110_vmmc1 = {
	.constraints = {
		.valid_modes_mask = REGULATOR_MODE_NORMAL
				| REGULATOR_MODE_STANDBY,
		.valid_ops_mask = REGULATOR_CHANGE_VOLTAGE
				| REGULATOR_CHANGE_MODE
				| REGULATOR_CHANGE_STATUS,
	},
};

/* VSIM for MMC1 pins DAT4..DAT7 (2 mA, plus card == max 50 mA) */
static struct regulator_init_data sbc8110_vsim = {
	.constraints = {
		.valid_modes_mask = REGULATOR_MODE_NORMAL
				| REGULATOR_MODE_STANDBY,
		.valid_ops_mask = REGULATOR_CHANGE_VOLTAGE
				| REGULATOR_CHANGE_MODE
				| REGULATOR_CHANGE_STATUS,
	},
};

/* VDAC for DSS driving S-Video (8 mA unloaded, max 65 mA) */
static struct regulator_init_data sbc8110_vdac = {
	.constraints = {
		.min_uV			= 1800000,
		.max_uV			= 1800000,
		.valid_modes_mask	= REGULATOR_MODE_NORMAL
					| REGULATOR_MODE_STANDBY,
		.valid_ops_mask		= REGULATOR_CHANGE_MODE
					| REGULATOR_CHANGE_STATUS,
	},
};

static int sbc8110_keymap[] = {
        KEY(0, 0, KEY_1),
        KEY(0, 1, KEY_2),
        KEY(0, 2, KEY_3),
        KEY(0, 3, KEY_F1),
        KEY(1, 0, KEY_4),
        KEY(1, 1, KEY_5),
        KEY(1, 2, KEY_6),
        KEY(1, 3, KEY_F2),
        KEY(2, 0, KEY_7),
        KEY(2, 1, KEY_8),
        KEY(2, 2, KEY_9),
        KEY(2, 3, KEY_ENTER),
        KEY(3, 0, KEY_F3),
        KEY(3, 1, KEY_0),
        KEY(3, 2, KEY_F4),
        KEY(3, 3, KEY_BACKSPACE)
};

static struct twl4030_keypad_data sbc8110_kp_data = {
        .rows           = 4,
        .cols           = 4,
        .keymap         = sbc8110_keymap,
        .keymapsize     = ARRAY_SIZE(sbc8110_keymap),
        .rep            = 1,
};

static struct twl4030_platform_data sbc8110_twldata = {
	.irq_base	= TWL4030_IRQ_BASE,
	.irq_end	= TWL4030_IRQ_END,

	/* platform_data for children goes here */
	.usb		= &sbc8110_usb_data,
	.gpio		= &sbc8110_gpio_data,
	.power		= GENERIC3430_T2SCRIPTS_DATA,
	.vmmc1		= &sbc8110_vmmc1,
	.vsim		= &sbc8110_vsim,
	.vdac		= &sbc8110_vdac,
	.keypad		= &sbc8110_kp_data,
};

static struct i2c_board_info __initdata sbc8110_i2c_boardinfo[] = {
	{
		I2C_BOARD_INFO("twl4030", 0x48),
		.flags = I2C_CLIENT_WAKE,
		.irq = INT_34XX_SYS_NIRQ,
		.platform_data = &sbc8110_twldata,
	},
};

static int __init omap3_sbc8110_i2c_init(void)
{
	omap_register_i2c_bus(1, 2600, sbc8110_i2c_boardinfo,
			ARRAY_SIZE(sbc8110_i2c_boardinfo));
	omap_register_i2c_bus(2, 400, NULL, 0);
	omap_register_i2c_bus(3, 400, NULL, 0);
	return 0;
}

#define OMAP_DM9000_GPIO_IRQ    25

static void __init omap_dm9000_init(void)
{
        if (gpio_request(OMAP_DM9000_GPIO_IRQ, "dm9000 irq") < 0) {
                printk(KERN_ERR "Failed to request GPIO%d for dm9000 IRQ\n",
                        OMAP_DM9000_GPIO_IRQ);
                return;
        }

        gpio_direction_input(OMAP_DM9000_GPIO_IRQ);
}

#define OMAP3_BEAGLE_TS_GPIO       27

static void ads7846_dev_init(void)
{
	if (gpio_request(OMAP3_BEAGLE_TS_GPIO, "ADS7846 pendown") < 0)
		printk(KERN_ERR "can't get ads7846 pen down GPIO\n");

	gpio_direction_input(OMAP3_BEAGLE_TS_GPIO);

	omap_set_gpio_debounce(OMAP3_BEAGLE_TS_GPIO, 1);
	omap_set_gpio_debounce_time(OMAP3_BEAGLE_TS_GPIO, 0xa);
}

static int ads7846_get_pendown_state(void)
{
	return !gpio_get_value(OMAP3_BEAGLE_TS_GPIO);
}

struct ads7846_platform_data ads7846_conf = {
	.x_max                  = 0x0fff,
	.y_max                  = 0x0fff,
//	.x_plate_ohms           = 180,
//	.pressure_max           = 255,
	.debounce_max           = 10,
	.debounce_tol           = 5,
	.debounce_rep           = 1,
	.get_pendown_state	= ads7846_get_pendown_state,
	.keep_vref_on		= 1,
	.settle_delay_usecs     = 150,
};

static struct omap2_mcspi_device_config ads7846_mcspi_config = {
	.turbo_mode	= 0,
	.single_channel	= 1,  /* 0: slave, 1: master */
};

struct spi_board_info omap3_sbc8110_spi_board_info[] = {
	[0] = {
		.modalias		= "ads7846",
		.bus_num		= 2,
		.chip_select		= 0,
		.max_speed_hz		= 1500000,
		.controller_data	= &ads7846_mcspi_config,
		.irq			= OMAP_GPIO_IRQ(OMAP3_BEAGLE_TS_GPIO),
		.platform_data		= &ads7846_conf,
	},
};

static void __init omap3_sbc8110_init_irq(void)
{
	omap2_init_common_hw(mt46h32m32lf6_sdrc_params);
	omap_init_irq();
	omap_gpio_init();
	omap_dm9000_init();
	ads7846_dev_init();
}

#ifdef CONFIG_OMAP2_DSS
#ifdef CONFIG_FB_OMAP2
static struct resource omap3evm_vout_resource[3 - CONFIG_FB_OMAP2_NUM_FBS] = {
};
#else /* CONFIG_FB_OMAP2 */
static struct resource omap3evm_vout_resource[2] = {
};
#endif /* CONFIG_FB_OMAP2 */
static struct platform_device omap3evm_vout_device = {
        .name                   = "omap_vout",
        .num_resources  = ARRAY_SIZE(omap3evm_vout_resource),
        .resource               = &omap3evm_vout_resource[0],
        .id             = -1,
};

static int lcd_enabled;
static int dvi_enabled;

static int omap3_evm_panel_enable_lcd(struct omap_display *display)
{
        if (dvi_enabled) {
                return -EINVAL;
        }

        twl4030_i2c_write_u8(TWL4030_MODULE_GPIO, 0x80, REG_GPIODATAOUT1);
        twl4030_i2c_write_u8(TWL4030_MODULE_LED, 0x0, 0x0);

        lcd_enabled = 1;
        return 0;
}

static void omap3_evm_panel_disable_lcd(struct omap_display *display)
{
        twl4030_i2c_write_u8(TWL4030_MODULE_GPIO, 0x00, REG_GPIODATAOUT1);
        twl4030_i2c_write_u8(TWL4030_MODULE_LED, 0x33, 0x0);

        lcd_enabled = 0;
}

static struct omap_display_data omap3_evm_display_data_lcd = {
        .type = OMAP_DISPLAY_TYPE_DPI,
        .name = "lcd",
        .panel_name = "panel-generic",
        .u.dpi.data_lines = 24,
        .panel_enable = omap3_evm_panel_enable_lcd,
        .panel_disable = omap3_evm_panel_disable_lcd,
};

static int omap3_evm_panel_enable_tv(struct omap_display *display)
{
        return 0;
}

static void omap3_evm_panel_disable_tv(struct omap_display *display)
{
}

static struct omap_display_data omap3_evm_display_data_tv = {
        .type = OMAP_DISPLAY_TYPE_VENC,
        .name = "tv",
        .u.venc.type = /*OMAP_DSS_VENC_TYPE_SVIDEO*/OMAP_DSS_VENC_TYPE_COMPOSITE,
        .panel_enable = omap3_evm_panel_enable_tv,
        .panel_disable = omap3_evm_panel_disable_tv,
};

static struct omap_dss_platform_data omap3_evm_dss_data = {
        .num_displays = 2,
        .displays = {
                &omap3_evm_display_data_lcd,
                &omap3_evm_display_data_tv,
        }
};
static struct platform_device omap3_evm_dss_device = {
        .name           = "omap-dss",
        .id             = -1,
        .dev            = {
                .platform_data = &omap3_evm_dss_data,
        },
};
#else
static struct platform_device omap3_sbc8110_lcd_device = {
	.name		= "omap3_sbc8110_lcd",
	.id		= -1,
};

static struct omap_lcd_config omap3_sbc8110_lcd_config __initdata = {
	.ctrl_name	= "internal",
};
#endif

static struct gpio_led gpio_leds[] = {
	{
		.name			= "led0",
		.default_trigger	= "heartbeat",
		.gpio			= 136,
		.active_low		= true,
	},
	{
		.name			= "led1",
		.default_trigger	= "mmc0",
		.gpio			= 137,
		.active_low		= true,
	},
	{
		.name			= "led2",
		.default_trigger	= "none",
		.gpio			= 138, 
		.active_low             = true,
	},
	{
		.name			= "led3",
		.default_trigger	= "none",
		.gpio			= 139,
		.active_low             = true,
	},
};

static struct gpio_led_platform_data gpio_led_info = {
	.leds		= gpio_leds,
	.num_leds	= ARRAY_SIZE(gpio_leds),
};

static struct platform_device leds_gpio = {
	.name	= "leds-gpio",
	.id	= -1,
	.dev	= {
		.platform_data	= &gpio_led_info,
	},
};

static struct gpio_keys_button gpio_buttons[] = {
	{
		.code			= BTN_EXTRA,
		.gpio			= 26,
		.desc			= "user",
		.wakeup			= 1,
	},
};

static struct gpio_keys_platform_data gpio_key_info = {
	.buttons	= gpio_buttons,
	.nbuttons	= ARRAY_SIZE(gpio_buttons),
};

static struct platform_device keys_gpio = {
	.name	= "gpio-keys",
	.id	= -1,
	.dev	= {
		.platform_data	= &gpio_key_info,
	},
};

#define OMAP_DM9000_BASE	0x2c000000
static struct resource omap_dm9000_resources[] = {
	[0] = {
		.start		= OMAP_DM9000_BASE,
		.end		= (OMAP_DM9000_BASE + 0x4 - 1),
		.flags		= IORESOURCE_MEM,
	},
	[1] = {
		.start		= (OMAP_DM9000_BASE + 0x400),
		.end		= (OMAP_DM9000_BASE + 0x400 + 0x4 - 1),
		.flags		= IORESOURCE_MEM,
	},
	[2] = {
		.start		= OMAP_GPIO_IRQ(OMAP_DM9000_GPIO_IRQ),
		.end		= OMAP_GPIO_IRQ(OMAP_DM9000_GPIO_IRQ),
		.flags		= IORESOURCE_IRQ | IRQF_TRIGGER_LOW,
	},
};

static struct dm9000_plat_data omap_dm9000_platdata = {
        .flags = DM9000_PLATF_16BITONLY,
};

static struct platform_device omap_dm9000_dev = {
	.name = "dm9000",
	.id = -1,
	.num_resources	= ARRAY_SIZE(omap_dm9000_resources),
	.resource	= omap_dm9000_resources,
        .dev = {
                .platform_data = &omap_dm9000_platdata,
        },
};

static struct omap_board_config_kernel omap3_sbc8110_config[] __initdata = {
	{ OMAP_TAG_UART,	&omap3_sbc8110_uart_config },
#ifndef	CONFIG_OMAP2_DSS
	{ OMAP_TAG_LCD,		&omap3_sbc8110_lcd_config },
#endif
};

static struct platform_device *omap3_sbc8110_devices[] __initdata = {
#ifdef CONFIG_OMAP2_DSS
	&omap3_evm_dss_device,
	&omap3evm_vout_device,
#else
	&omap3_sbc8110_lcd_device,
#endif
	&leds_gpio,
	&keys_gpio,
	&omap_dm9000_dev,
};

static void __init omap3_sbc8110_flash_init(void)
{
	u8 cs = 0;
	u8 nandcs = GPMC_CS_NUM + 1;

	u32 gpmc_base_add = OMAP34XX_GPMC_VIRT;

	/* find out the chip-select on which NAND exists */
	while (cs < GPMC_CS_NUM) {
		u32 ret = 0;
		ret = gpmc_cs_read_reg(cs, GPMC_CS_CONFIG1);

		if ((ret & 0xC00) == 0x800) {
			printk(KERN_INFO "Found NAND on CS%d\n", cs);
			if (nandcs > GPMC_CS_NUM)
				nandcs = cs;
		}
		cs++;
	}

	if (nandcs > GPMC_CS_NUM) {
		printk(KERN_INFO "NAND: Unable to find configuration "
				 "in GPMC\n ");
		return;
	}

	if (nandcs < GPMC_CS_NUM) {
		omap3_sbc8110_nand_data.cs = nandcs;
		omap3_sbc8110_nand_data.gpmc_cs_baseaddr = (void *)
			(gpmc_base_add + GPMC_CS0_BASE + nandcs * GPMC_CS_SIZE);
		omap3_sbc8110_nand_data.gpmc_baseaddr = (void *) (gpmc_base_add);

		printk(KERN_INFO "Registering NAND on CS%d\n", nandcs);
		if (platform_device_register(&omap3_sbc8110_nand_device) < 0)
			printk(KERN_ERR "Unable to register NAND device\n");
	}
}

static void __init omap3_sbc8110_init(void)
{
	omap3_sbc8110_i2c_init();
	platform_add_devices(omap3_sbc8110_devices,
			ARRAY_SIZE(omap3_sbc8110_devices));
	omap_board_config = omap3_sbc8110_config;
	omap_board_config_size = ARRAY_SIZE(omap3_sbc8110_config);

	spi_register_board_info(omap3_sbc8110_spi_board_info,
                                ARRAY_SIZE(omap3_sbc8110_spi_board_info));

	omap_serial_init();

	omap_cfg_reg(J25_34XX_GPIO170);
	gpio_request(170, "DVI_nPD");
	/* REVISIT leave DVI powered down until it's needed ... */
	gpio_direction_output(170, true);

	usb_musb_init();
	usb_ehci_init();
	omap3_sbc8110_flash_init();
}

static void __init omap3_sbc8110_map_io(void)
{
	omap2_set_globals_343x();
	omap2_map_common_io();
}

MACHINE_START(OMAP3_SBC8110, "OMAP3 SBC8110 Board")
	.phys_io	= 0x48000000,
	.io_pg_offst	= ((0xd8000000) >> 18) & 0xfffc,
	.boot_params	= 0x80000100,
	.map_io		= omap3_sbc8110_map_io,
	.init_irq	= omap3_sbc8110_init_irq,
	.init_machine	= omap3_sbc8110_init,
	.timer		= &omap_timer,
MACHINE_END
