/*
 * character device wrapper for generic gpio layer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA02111-1307USA
 *
 * Feedback, Bugs...  blogic@openwrt.org
 *
 */

#include <linux/module.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <asm/uaccess.h>
#include <asm/io.h>
#include <asm/gpio.h>
#include <asm/atomic.h>
#include <linux/init.h>
#include <linux/genhd.h>
#include <linux/miscdevice.h>

#define GPIO_BEEP	159

static int beep_val;

static ssize_t beep_read(struct file *filp, char *buf,size_t count,loff_t *f_ops)
{
	return count;
}

static ssize_t beep_write(struct file *filp,const char *buf,size_t count,loff_t *f_ops)
{
	return count;
}

static int beep_ioctl(struct inode * inode, struct file * file, unsigned int cmd, unsigned long arg)
{
	return 0;
}

static int beep_open(struct inode *inode, struct file *file)
{
	return 0;
}

static int beep_close(struct inode * inode, struct file * file)
{
	return 0;
}

static ssize_t beep_val_show(struct device *dev, struct device_attribute *attr, char *buf)
{
	return sprintf(buf, "%u\n", beep_val);
}

static ssize_t beep_val_store(struct device *dev, struct device_attribute *attr, const char *buf, size_t size)
{
	beep_val = simple_strtoul(buf, NULL, 10);
	gpio_direction_output(GPIO_BEEP, beep_val);

	return size;
}

struct file_operations beep_fops = {
	.read		= beep_read,
	.write		= beep_write,	
	.ioctl		= beep_ioctl,
	.open		= beep_open,
	.release	= beep_close,
};

static struct miscdevice beep_dev = {
        .minor         = MISC_DYNAMIC_MINOR,
        .name         = "beep",                   
        .fops         = &beep_fops,
};

static DEVICE_ATTR(val, 0644, beep_val_show, beep_val_store);

static int __init beep_dev_init(void)
{
	int ret;

	ret = misc_register(&beep_dev);
	if(ret){
		printk(KERN_ERR "misc_register failed\n");
		return ret;
	}

        ret = device_create_file(beep_dev.this_device, &dev_attr_val);
        if (ret){
		printk(KERN_ERR "creat attr file failed\n");
		misc_deregister(&beep_dev);
		return ret;
	}

        if (gpio_request(GPIO_BEEP, "BEEP") < 0) {
                printk(KERN_ERR ": Can't get GPIO %d\n", GPIO_BEEP);
                return -EINVAL;
        }

        gpio_direction_output(GPIO_BEEP, 0);
	
	return 0;
}

static void __exit beep_dev_exit(void)
{
	device_remove_file(beep_dev.this_device, &dev_attr_val);
	misc_deregister(&beep_dev);
}

module_init (beep_dev_init);
module_exit (beep_dev_exit);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Timll");
MODULE_DESCRIPTION("Character device for beep dev api");
