/*
 * (C) Copyright 2006
 * Texas Instruments.
 * Richard Woodruff <r-woodruff2@ti.com>
 * Syed Mohammed Khasim <x0khasim@ti.com>
 *
 * Configuration settings for the 3430 TI SDP3430 board.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef __CONFIG_H
#define __CONFIG_H

/*
 * High Level Configuration Options
 */
#define CONFIG_ARMCORTEXA8	1	/* This is an ARM V7 CPU core */
#define CONFIG_OMAP		1	/* in a TI OMAP core */
#define CONFIG_OMAP34XX		1	/* which is a 34XX */
#define CONFIG_OMAP3430		1	/* which is in a 3430 */
#define CONFIG_OMAP3_SBC8110	1	/* working with SBC8110 */
#define CONFIG_DOS_PARTITION	1

#define CONFIG_MTD_DEBUG
#define CONFIG_MTD_DEBUG_VERBOSE 1

#include <asm/arch/cpu.h>	/* get chip and board defs */

/* Clock Defines */
#define V_OSCK                   26000000	/* Clock output from T2 */
#define V_SCLK                   (V_OSCK >> 1)

//#define PRCM_CLK_CFG2_266MHZ   1	/* VDD2=1.15v - 133MHz DDR */
#define PRCM_CLK_CFG2_332MHZ     1    /* VDD2=1.15v - 166MHz DDR */			
#define PRCM_PCLK_OPP2           1	/* ARM=381MHz - VDD1=1.20v */

#undef CONFIG_USE_IRQ		/* no support for IRQs */
#define CONFIG_MISC_INIT_R								

#define CONFIG_CMDLINE_TAG       1	/* enable passing of ATAGs */
#define CONFIG_SETUP_MEMORY_TAGS 1
#define CONFIG_INITRD_TAG        1
#define CONFIG_REVISION_TAG      1

/*
 * Size of malloc() pool
 */
#define CFG_ENV_SIZE             SZ_128K	/* Total Size Environment Sector */
#define CFG_MALLOC_LEN           (CFG_ENV_SIZE + SZ_128K)
#define CFG_GBL_DATA_SIZE        128	/* bytes reserved for initial data */

/*
 * Hardware drivers
 */

/*
 * DM9000
 */
#define CONFIG_DRIVER_DM9000    1
#define CONFIG_DM9000_BASE      0x2c000000
#define DM9000_IO               CONFIG_DM9000_BASE
#define DM9000_DATA             (CONFIG_DM9000_BASE+0x400)
#define CONFIG_DM9000_USE_16BIT

/*
 * NS16550 Configuration
 */
#define V_NS16550_CLK            (48000000)	/* 48MHz (APLL96/2) */

#define CFG_NS16550
#define CFG_NS16550_SERIAL
#define CFG_NS16550_REG_SIZE     (-4)
#define CFG_NS16550_CLK          V_NS16550_CLK

/*
 * select serial console configuration
 */
#define CONFIG_CONS_INDEX        3
#define CFG_NS16550_COM3         OMAP34XX_UART3

/* allow to overwrite serial and ethaddr */
#define CONFIG_BAUDRATE          115200
#define CFG_BAUDRATE_TABLE       {4800, 9600, 19200, 38400, 57600, 115200}

/*
 * MMC
 */
#define CONFIG_MMC		1
#define CFG_MMC_BASE		0xF0000000
#define CONFIG_DOS_PARTITION	1

/*
 * I2C
 */
#define CFG_I2C_SPEED            100
#define CFG_I2C_SLAVE            1
#define CFG_I2C_BUS              0
#define CFG_I2C_BUS_SELECT       1
#define CONFIG_DRIVER_OMAP34XX_I2C 1

/* commands to include */

#define CONFIG_CMD_NET
#define CONFIG_CMD_PING
#define CONFIG_CMD_EXT2		/* EXT2 Support			*/
#define CONFIG_CMD_FAT		/* FAT support			*/
#define CONFIG_CMD_JFFS2	/* JFFS2 Support		*/

#define CONFIG_CMD_I2C		/* I2C serial bus support	*/
#define CONFIG_CMD_MMC		/* MMC support			*/
#define CONFIG_CMD_NAND		/* NAND support			*/

#define CONFIG_CMD_AUTOSCRIPT	/* autoscript support		*/
#define CONFIG_CMD_BDI		/* bdinfo			*/
#define CONFIG_CMD_BOOTD	/* bootd			*/
#define CONFIG_CMD_CONSOLE	/* coninfo			*/
#define CONFIG_CMD_ECHO		/* echo arguments		*/
#define CONFIG_CMD_ENV		/* saveenv			*/
#define CONFIG_CMD_ITEST	/* Integer (and string) test	*/
#define CONFIG_CMD_LOADB	/* loadb			*/
#define CONFIG_CMD_MEMORY	/* md mm nm mw cp cmp crc base loop mtest */
#define CONFIG_CMD_MISC		/* misc functions like sleep etc*/
#define CONFIG_CMD_RUN		/* run command in env variable	*/

/* Environment information */
#define CONFIG_BOOTDELAY         3

/* Un comment the below CONFIG_BOOTCMD macro to auto flash u-boot onto new board */
//#define CONFIG_BOOTCOMMAND	 \
	"nand erase;mmcinit;\
	fatload mmc 0 80000000 x-load.bin.ift_for_NAND;nand ecc hw;nand write.i 80000000 0 $(filesize);\
	fatload mmc 0 80000000 flash-uboot.bin;nand ecc sw;nand write.i 80000000 80000 $(filesize);\
	fatload mmc 0 80000000 uImage;nand write.i 80000000 280000 $(filesize);\
	fatload mmc 0 81000000 ubi.img;nand write.i 81000000 680000 $(filesize);\
	led flash all"

/* for ubifs */	
//#define CONFIG_BOOTARGS "console=ttyS2,115200n8 ubi.mtd=4 root=ubi0:rootfs rootfstype=ubifs video=omapfb:mode:720p60"
//#define CONFIG_BOOTCOMMAND "nand read.i 80300000 280000 210000\;bootm 80300000"

/* for ramdisk */
//#define CONFIG_BOOTARGS "console=ttyS2,115200n8 root=/dev/ram0 rw rootfstype=ext2 initrd=0x81600000,40M video=omapfb:mode:720p60"
//#define CONFIG_BOOTCOMMAND \
        "mmcinit;fatload mmc 0 0x80300000 uImage; fatload mmc 0 0x81600000 ramdisk.gz; bootm 0x80300000"

#define CONFIG_ETHADDR          aa:bb:cc:dd:ee:ff
#define CONFIG_IPADDR           192.192.192.200
#define CONFIG_SERVERIP         192.192.192.90
#define CONFIG_GATEWAYIP        192.192.192.101
#define CONFIG_NETMASK          255.255.255.0

#define CONFIG_BOOTFILE          "uImage"
#define CONFIG_AUTO_COMPLETE     1
/*
 * Miscellaneous configurable options
 */
#define V_PROMPT                 "OMAP3 SBC8110 # "

#define CFG_LONGHELP		/* undef to save memory */
#define CFG_PROMPT               V_PROMPT
#define CFG_CBSIZE               512	/* Console I/O Buffer Size */
/* Print Buffer Size */
#define CFG_PBSIZE               (CFG_CBSIZE+sizeof(CFG_PROMPT)+16)
#define CFG_MAXARGS              16	/* max number of command args */
#define CFG_BARGSIZE             CFG_CBSIZE	/* Boot Argument Buffer Size */

#define CFG_MEMTEST_START        (OMAP34XX_SDRC_CS0)	/* memtest works on */
#define CFG_MEMTEST_END          (OMAP34XX_SDRC_CS0+SZ_31M)

#undef	CFG_CLKS_IN_HZ		/* everything, incl board info, in Hz */

#define CFG_LOAD_ADDR            (OMAP34XX_SDRC_CS0)	/* default load address */

/* 2430 has 12 GP timers, they can be driven by the SysClk (12/13/19.2) or by
 * 32KHz clk, or from external sig. This rate is divided by a local divisor.
 */
#define V_PVT                    7

#define CFG_TIMERBASE            OMAP34XX_GPT2
#define CFG_PVT                  V_PVT	/* 2^(pvt+1) */
#define CFG_HZ                   ((V_SCLK)/(2 << CFG_PVT))

/*-----------------------------------------------------------------------
 * Stack sizes
 *
 * The stack sizes are set up in start.S using the settings below
 */
#define CONFIG_STACKSIZE	SZ_128K	/* regular stack */
#ifdef CONFIG_USE_IRQ
#define CONFIG_STACKSIZE_IRQ	SZ_4K	/* IRQ stack */
#define CONFIG_STACKSIZE_FIQ	SZ_4K	/* FIQ stack */
#endif

/*-----------------------------------------------------------------------
 * Physical Memory Map
 */
#define CONFIG_NR_DRAM_BANKS	2	/* CS1 may or may not be populated */
#define PHYS_SDRAM_1		OMAP34XX_SDRC_CS0
#define PHYS_SDRAM_1_SIZE	SZ_32M	/* at least 32 meg */
#define PHYS_SDRAM_2		OMAP34XX_SDRC_CS1

/* SDRAM Bank Allocation method */
#define SDRC_R_B_C		1

/*-----------------------------------------------------------------------
 * FLASH and environment organization
 */

/* **** PISMO SUPPORT *** */

/* Configure the PISMO */
#define PISMO1_NOR_SIZE_SDPV2	GPMC_SIZE_128M
#define PISMO1_NOR_SIZE		GPMC_SIZE_64M

#define PISMO1_NAND_SIZE	GPMC_SIZE_128M
#define PISMO1_ONEN_SIZE	GPMC_SIZE_128M
#define DBG_MPDB_SIZE		GPMC_SIZE_16M
#define PISMO2_SIZE		0

#define CFG_NO_FLASH

#define CFG_MAX_FLASH_SECT	(520)	/* max number of sectors on one chip */
#define CFG_MAX_FLASH_BANKS      2	/* max number of flash banks */
#define CFG_MONITOR_LEN		SZ_256K	/* Reserve 2 sectors */

#define PHYS_FLASH_SIZE_SDPV2	SZ_128M
#define PHYS_FLASH_SIZE		SZ_32M

#define CFG_FLASH_BASE		boot_flash_base
#define PHYS_FLASH_SECT_SIZE	boot_flash_sec
/* Dummy declaration of flash banks to get compilation right */
#define CFG_FLASH_BANKS_LIST	{0, 0}

#define CFG_MONITOR_BASE	CFG_FLASH_BASE	/* Monitor at start of flash */
#define CFG_ONENAND_BASE	ONENAND_MAP

#define CFG_ENV_IS_IN_NAND	1
#define ONENAND_ENV_OFFSET	0x260000	/* environment starts here  */
#define SMNAND_ENV_OFFSET	0x260000	/* environment starts here  */

#define CFG_ENV_SECT_SIZE	boot_flash_sec
#define CFG_ENV_OFFSET		boot_flash_off
#define CFG_ENV_ADDR		SMNAND_ENV_OFFSET

#define CONFIG_ENV_OVERWRITE
#define ENV_IS_VARIABLE         1

/*-----------------------------------------------------------------------
 * CFI FLASH driver setup
 */
/* timeout values are in ticks */
#define CFG_FLASH_ERASE_TOUT	(100*CFG_HZ)	/* Timeout for Flash Erase */
#define CFG_FLASH_WRITE_TOUT	(100*CFG_HZ)	/* Timeout for Flash Write */

/* Flash banks JFFS2 should use */
#define CFG_MAX_MTD_BANKS	(CFG_MAX_FLASH_BANKS+CFG_MAX_NAND_DEVICE)
#define CFG_JFFS2_MEM_NAND
#define CFG_JFFS2_FIRST_BANK	CFG_MAX_FLASH_BANKS	/* use flash_info[2] */
#define CFG_JFFS2_NUM_BANKS	1

/*
 *  *  Board NAND Info.
 *   */
#define CFG_NAND_ADDR NAND_BASE /* physical address to access nand */
#define CFG_NAND_BASE NAND_BASE /* physical address to access nand at CS0 */
#define CFG_NAND_WIDTH_16

#define CFG_MAX_NAND_DEVICE      1      /* Max number of NAND devices */
#define SECTORSIZE               512

#define NAND_ALLOW_ERASE_ALL
#define ADDR_COLUMN              1
#define ADDR_PAGE                2
#define ADDR_COLUMN_PAGE         3

#define NAND_ChipID_UNKNOWN      0x00
#define NAND_MAX_FLOORS          1
#define NAND_MAX_CHIPS           1
#define NAND_NO_RB               1
#define CFG_NAND_WP

#define CONFIG_JFFS2_NAND
#define CONFIG_JFFS2_DEV                "nand0"         /* nand device jffs2 lives on */
#define CONFIG_JFFS2_PART_OFFSET        0x680000        /* start of jffs2 partition */
#define CONFIG_JFFS2_PART_SIZE          0xf980000       /* size of jffs2 partition */

#define WRITE_NAND_COMMAND(d, adr) __raw_writew(d, (nand_cs_base + GPMC_NAND_CMD))
#define WRITE_NAND_ADDRESS(d, adr) __raw_writew(d, (nand_cs_base + GPMC_NAND_ADR))
#define WRITE_NAND(d, adr) __raw_writew(d, (nand_cs_base + GPMC_NAND_DAT))
#define READ_NAND(adr) __raw_readw((nand_cs_base + GPMC_NAND_DAT))

/* Other NAND Access APIs */
#define NAND_WP_OFF()  do {*(volatile u32 *)(GPMC_CONFIG) |= 0x00000010;} while(0)
#define NAND_WP_ON()  do {*(volatile u32 *)(GPMC_CONFIG) &= ~0x00000010;} while(0)
#define NAND_DISABLE_CE(nand)
#define NAND_ENABLE_CE(nand)
#define NAND_WAIT_READY(nand)	udelay(10)

#ifndef __ASSEMBLY__
extern unsigned int nand_cs_base;
extern unsigned int boot_flash_base;
extern volatile unsigned int boot_flash_env_addr;
extern unsigned int boot_flash_off;
extern unsigned int boot_flash_sec;
extern unsigned int boot_flash_type;
#endif

#endif				/* __CONFIG_H */
